#include "StdAfx.h"
#include "Hand.h"
#include "Deck.h"
#include "Card.h"
#include <vector>
#include <iostream>
#include "HandClass.h"
#include <algorithm>

//Hand::Hand(void)
//{
//}


Hand::~Hand(void)
{
}

//just adds a card to the hand
void Hand::addACard(Card theCard)
{
	if( theCards.size() < 5 )
	{
		theCards.push_back(theCard);
	}
	else
	{
		cout << "Adding card to a full hand";
	}
}

//swaps out a particular card
void Hand::swapCard(Deck& theDeck, int handSpotToSwap)
{
	Card tempCard;
	theCards[handSpotToSwap] = theDeck.dealTopCard();
}

Card tempCard;
void Hand::printToCL()
{
	for(std::vector<Card>::iterator it=theCards.begin(); it != theCards.end(); ++it)
	{
		//for each card in the hand, print the crd to the command line
		tempCard = *it;
		tempCard.printToCL();
		cout << "  ";
	}
	
}


//clears a hand
void Hand::clear()
{
	theCards.clear();
}


vector<Card>& Hand::getTheCards()
{
	return theCards;
}



//just returns the list of hand classifications found
vector<HandClass>& Hand::getHandClasses()
{
	return handClasses;
}


//compares two cards by rank
bool cardCompareByRank(Card card1, Card card2)
{
	return card1.getRank() < card2.getRank();
}


bool Hand::isItAStraight()
{
	HandClass tempClass;		
	tempClass.classPositions[0] = 0;
	tempClass.classPositions[1] = 1;
	tempClass.classPositions[2] = 2;
	tempClass.classPositions[3] = 3;
	tempClass.classPositions[4] = 4;
	bool isStraight = true;
	//sort and then decide if it is a straight
	std::sort(theCards.begin(),theCards.end(), cardCompareByRank); 
	for(int i = 1 ; i < 5 ; i++)
	{
		if ( theCards[i].getRank() - 1 != theCards[i-1].getRank()  )
		{
			isStraight = false;
			break;
		}
	}
	if(isStraight)
	{
		//load a straight
		tempClass.setLoaded();
		tempClass.setClassification(HandClass::STRAIGHT);
		handClasses[HandClass::STRAIGHT] = tempClass;
	}
	return isStraight;
}


//finds if it is a straight flush
bool Hand::istItAStraightFlush()
{
	if(handClasses[HandClass::STRAIGHT].isItLoaded() && handClasses[HandClass::FLUSH].isItLoaded())
	{
			HandClass tempClass;		
			tempClass.classPositions[0] = 0;
			tempClass.classPositions[0] = 1;
			tempClass.classPositions[0] = 2;
			tempClass.classPositions[0] = 3;
			tempClass.classPositions[0] = 4;
			tempClass.setLoaded();
			tempClass.setClassification(HandClass::STRAIGHT_FLUSH);
			handClasses[HandClass::STRAIGHT_FLUSH] = tempClass;
			return true;
	}
	else
	{
		return false;
	}
}


//this method finds all the hand types (called here classes) within a hand and loads them into the list of hand classes.
void Hand::establishHandClasses()
{
	//maybe reset the hand classes?
	initializeHandClasses();
	findHighCard();
	findNofAKind();
	isItAStraight();
	isItAFlush();
	istItAStraightFlush();
}

//decides if it is a flush
bool Hand::isItAFlush()
{
	bool isFlush = true;
	Card::suits_t theSuit;
	theSuit = theCards[0].getSuit();
	for(int i = 1; i < 5 ; i++)
	{
		if(theCards[i].getSuit() != theSuit)
		{
			isFlush = false;
		}
	}
	if(isFlush)
	{
		handClasses[HandClass::FLUSH].setClassification(HandClass::FLUSH);
		handClasses[HandClass::FLUSH].setLoaded();
	}
	return isFlush;
}

int rankPos;
int tempRank;
Card::suits_t tempSuit;
//finds the high card
Card Hand::findHighCard()
{
	HandClass tempClass;
	tempClass.setClassification(HandClass::HIGH_CARD);
	tempRank = theCards[0].getRank();
	tempSuit = theCards[0].getSuit();
	rankPos = 0;
	//do nothing
	for(int i = 0 ; i < 5 ; i++)
	{
		if(tempRank < theCards[i].getRank() && tempSuit < theCards[i].getSuit())
		{
			tempRank = theCards[i].getRank();
			tempSuit = theCards[i].getSuit();
			rankPos = i;
		}
	}
	tempClass.classPositions[0] = rankPos;
	handClasses[HandClass::HIGH_CARD] = tempClass;
	return theCards[rankPos];
}



int swap1, swap2, swap3;
//this method will find all instances of n of akind for n from 2 to 4.  As it finds these, it loads
void Hand::findNofAKind()
{

	HandClass tempClass;
	vector<int> cardPositions;
	for(int i = 0 ; i < 5 ; i++)
	{
		cardPositions.clear();
		cardPositions.push_back(i);
		for(int j = 0 ; j < 5 ; j++)
		{
				if( (j != i) && (theCards[i].getRank() == theCards[j].getRank()))  //find if two ranks are the same
				{
					if( j < i )  //if we have already found this pair, then end the search and move on
					{
						break;
					}
					else  //record the position of the matching card
					{
						cardPositions.push_back(j);
					}
				}
		}
		//load the positions into tempClass
		for(int k = 0 ; k < cardPositions.size() ; k++)
		{
			tempClass.classPositions[k] = cardPositions[k];
		}
		switch( cardPositions.size())
		{
			case 1:
				//high card is already set so do nothing
				break;
			case 2:
				// move this into a small method of its own.
				//check to see if there is already a pair.  If not, then load hand classes vector with a pair.  If there is already a pair, then load hand classes vector with two-pair.
				tempClass.setClassification(HandClass::PAIR);
				tempClass.setLoaded();
				if(handClasses[HandClass::PAIR].isItLoaded())  //check to see if there is already a pair
				{
					//there is already a pair so, first create a twopair handclass given the two pairs you have found and then load two of a kind	
					tempClass.setClassification(HandClass::TWO_PAIR);
					//now load the card positions
					tempClass.classPositions[2] = handClasses[HandClass::PAIR].classPositions[0];
					tempClass.classPositions[3] = handClasses[HandClass::PAIR].classPositions[1];
					handClasses[HandClass::TWO_PAIR] = tempClass;
				}
				else
				{
					handClasses[HandClass::PAIR] = tempClass;
				}
				if (handClasses[HandClass::THREE_KIND].isItLoaded())  //check to see if there is three of a kind
				{
					//load full house
					tempClass.setClassification(HandClass::FULL_HOUSE);
					tempClass.setLoaded();
					tempClass.classPositions[2] = handClasses[HandClass::THREE_KIND].classPositions[0];
					tempClass.classPositions[3] = handClasses[HandClass::THREE_KIND].classPositions[1];
					tempClass.classPositions[4] = handClasses[HandClass::THREE_KIND].classPositions[2];
					handClasses[HandClass::FULL_HOUSE] = tempClass;
				}
				break;
			case 3:
				//check if there is a pair, if so, then set to full house
				//load hand classes vector with a triple
				tempClass.setClassification(HandClass::THREE_KIND);
				tempClass.setLoaded();
				handClasses[HandClass::THREE_KIND] = tempClass;
				if(handClasses[HandClass::PAIR].isItLoaded())
				{
					//load a full house
					tempClass.setClassification(HandClass::FULL_HOUSE);
					tempClass.setLoaded();
					swap1 = tempClass.classPositions[0];
					swap2 = tempClass.classPositions[1];
					swap3 = tempClass.classPositions[2];
					tempClass.classPositions[0] = handClasses[HandClass::PAIR].classPositions[0];
					tempClass.classPositions[1] = handClasses[HandClass::PAIR].classPositions[1];
					tempClass.classPositions[2] = swap1;
					tempClass.classPositions[3] = swap2;
					tempClass.classPositions[4] = swap3;
					handClasses[HandClass::FULL_HOUSE] = tempClass;
				}
				break;
			case 4:
				//load hand classes vector with 4 of a kind
				tempClass.setClassification(HandClass::FOUR_KIND);
				tempClass.setLoaded();
				handClasses[HandClass::FOUR_KIND] = tempClass;
				break;
		}
	}
}


//sets the vector of hand classes to an initialized state
void Hand::initializeHandClasses()
{
	vector<HandClass> tempVec(20, HandClass());
	handClasses = tempVec;
}


